@@ -6,7 +6,8 @@ import codecs
 import fnmatch
 import logging
 import itertools
-from utils import PageNotFoundError, NamespaceNotFoundError
+from utils import (PageNotFoundError, NamespaceNotFoundError,
+        split_page_url)
 
 
 META_ENDPOINT = '_meta'
@@ -15,6 +16,9 @@ META_ENDPOINT = '_meta'
 logger = logging.getLogger(__name__)
 
 
+valid_filename_pattern = re.compile('^[\w \.\-\(\)\[\]\\/]+$', re.UNICODE)
+
+
 class PageInfo(object):
     def __init__(self, url, path):
         self.url = url
@@ -34,14 +38,14 @@ class FileSystem(object):
         file-system paths, and for scanning the file-system
         to list existing pages.
     """
-    def __init__(self, root):
+    def __init__(self, root, config):
         self.root = unicode(root)
 
         self.excluded = None
         self.page_extensions = None
-        self.default_extension = '.txt'
+        self.default_extension = config.get('wiki', 'default_extension')
 
-    def initFs(self, wiki):
+    def start(self, wiki):
         self.page_extensions = list(set(
             itertools.chain(*wiki.formatters.itervalues())))
 
@@ -50,13 +54,18 @@ class FileSystem(object):
         excluded += wiki.scm.getSpecialFilenames()
         self.excluded = [os.path.join(self.root, e) for e in excluded]
 
-        self.default_extension = wiki.config.get('wiki', 'default_extension')
+    def init(self, wiki):
+        pass
+
+    def postInit(self):
+        pass
 
     def getPageInfos(self, subdir=None):
         basepath = self.root
         if subdir is not None:
             basepath = self.getPhysicalNamespacePath(subdir)
 
+        logger.debug("Scanning for pages in: %s" % basepath)
         for dirpath, dirnames, filenames in os.walk(basepath):
             incl_dirnames = []
             for d in dirnames:
@@ -74,6 +83,7 @@ class FileSystem(object):
                     yield page_info
 
     def getPageInfo(self, path):
+        logger.debug("Reading page info from: %s" % path)
         if not isinstance(path, unicode):
             path = unicode(path)
         for e in self.excluded:
@@ -81,18 +91,23 @@ class FileSystem(object):
                 return None
         return self._getPageInfo(path)
 
-    def getPage(self, url):
+    def findPageInfo(self, url):
+        logger.debug("Searching for page: %s" % url)
         path = self.getPhysicalPagePath(url)
         return PageInfo(url, path)
 
     def setPage(self, url, content):
         path = self.getPhysicalPagePath(url, make_new=True)
         logger.debug("Saving page '%s' to: %s" % (url, path))
+        dirname = os.path.dirname(path)
+        if not os.path.isdir(dirname):
+            os.makedirs(dirname, 0775)
         with codecs.open(path, 'w', encoding='utf-8') as f:
             f.write(content)
         return PageInfo(url, path)
 
     def pageExists(self, url):
+        logger.debug("Searching for page: %s" % url)
         try:
             self.getPhysicalPagePath(url)
             return True
@@ -107,6 +122,7 @@ class FileSystem(object):
 
     def _getPageInfo(self, path):
         meta = None
+        abs_path = os.path.abspath(path)
         rel_path = os.path.relpath(path, self.root)
         if rel_path.startswith(META_ENDPOINT + os.sep):
             rel_path = rel_path[len(META_ENDPOINT) + 1:]
@@ -122,15 +138,10 @@ class FileSystem(object):
         url = '/' + name
         if meta:
             url = u"%s:/%s" % (meta.lower(), name)
-        return PageInfo(url, path)
+        return PageInfo(url, abs_path)
 
     def _getPhysicalPath(self, url, is_file=True, make_new=False):
-        endpoint = None
-        m = re.match(r'(\w[\w\d]+)\:(.*)', url)
-        if m:
-            endpoint = str(m.group(1))
-            url = str(m.group(2)).strip()
-
+        endpoint, url = split_page_url(url)
         if url[0] != '/':
             raise ValueError("Page URLs need to be absolute: " + url)
         if string.find(url, '..') >= 0:
@@ -144,15 +155,23 @@ class FileSystem(object):
 
         # Make the URL into a relative file-system path.
         url_path = url[1:].replace('/', os.sep)
+        if url_path[0] == os.sep:
+            raise ValueError("Page URLs can only have one slash at the "
+                    "beginning. Got: %s" % url)
 
         # If we want a non-existing file's path, just build that.
         if make_new:
+            if (url_path[-1] == os.sep or
+                    not valid_filename_pattern.match(url_path)):
+                raise ValueError("Invalid URL: %s" % url_path)
             return os.path.join(root, url_path + '.' + self.default_extension)
 
         # Find the right file-system entry for this URL.
         url_path = os.path.join(root, url_path)
         if is_file:
             dirname, basename = os.path.split(url_path)
+            if basename == '':
+                raise ValueError("Invalid URL: %s" % url_path)
             if not os.path.isdir(dirname):
                 self._throwNotFoundError(url, root, is_file)
 
