@@ -3,7 +3,7 @@ from flask import g, abort, request, jsonify
 from flask.ext.login import current_user
 from wikked.page import Page, PageData
 from wikked.formatter import PageFormatter, FormattingContext
-from wikked.views import (make_page_title, make_auth_response, get_page_or_none,
+from wikked.views import (make_page_title, get_page_or_none,
         is_page_writable, get_page_meta, url_from_viewarg,
         split_url_from_viewarg)
 from wikked.web import app
@@ -17,7 +17,7 @@ class DummyPage(Page):
         self._text = text
 
     def _loadData(self):
-        extension = self.wiki.config.get('wiki', 'default_extension')
+        extension = self.wiki.fs.default_extension
         data = PageData()
         data.path = '__preview__.' + extension
         data.filename = '__preview__'
@@ -30,7 +30,10 @@ class DummyPage(Page):
         data.local_meta = ctx.meta
         data.local_links = ctx.out_links
 
-        data.title = make_page_title(self.url)
+        data.title = (data.local_meta.get('title') or
+                make_page_title(self.url))
+        if isinstance(data.title, list):
+            data.title = data.title[0]
 
         return data
 
@@ -58,7 +61,7 @@ def get_edit_page(url, default_title=None, custom_data=None):
             }
     if custom_data:
         result.update(custom_data)
-    return make_auth_response(result)
+    return jsonify(result)
 
 
 def do_edit_page(url, default_message):
@@ -85,7 +88,7 @@ def do_edit_page(url, default_message):
     g.wiki.setPage(url, page_fields)
 
     result = {'saved': 1}
-    return make_auth_response(result)
+    return jsonify(result)
 
 
 @app.route('/api/edit/', methods=['GET', 'POST'])
@@ -142,3 +145,20 @@ def api_rename_page(url):
 def api_delete_page(url):
     pass
 
+
+@app.route('/api/validate/newpage', methods=['GET', 'POST'])
+def api_validate_newpage():
+    path = request.form.get('title')
+    if path is None:
+        abort(400)
+    path = url_from_viewarg(path)
+    try:
+        # Check that there's no page with that name already, and that
+        # the name can be correctly mapped to a filename.
+        if g.wiki.pageExists(path):
+            raise Exception("Page '%s' already exists" % path)
+        g.wiki.fs.getPhysicalPagePath(path, make_new=True)
+    except Exception:
+        return '"This page name is invalid or unavailable"'
+    return '"true"'
+
